@@ -1,7 +1,7 @@
 /*
 
   Copyright (C) 1998   Andreas Wüst (AndreasWuest@gmx.de)
-  Copyright (C) 2006-2007   Dmitry Suzdalev (dimsuz@gmail.com)
+  Copyright (C) 2006-2009   Dmitry Suzdalev (dimsuz@gmail.com)
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -34,9 +34,10 @@
 #include <kconfig.h>
 #include <kglobalsettings.h>
 #include <kfiledialog.h>
+#include <kdebug.h>
 
-GameWidget::GameWidget ( int startingLevel, QWidget *parent )
-    : QWidget( parent ), m_allowAnyLevelSwitch( false ), m_moves(0)
+GameWidget::GameWidget ( const QString& levelSet, QWidget *parent )
+    : QWidget( parent ), m_allowAnyLevelSwitch( false ), m_moves(0), m_level(0)
 {
     m_highscore = new KAtomicHighscores();
     m_levelHighscore = 0;
@@ -56,8 +57,11 @@ GameWidget::GameWidget ( int startingLevel, QWidget *parent )
     m_view->setFrameStyle( QFrame::NoFrame );
     m_view->setCacheMode( QGraphicsView::CacheBackground );
 
+    // TODO uncomment DontSavePainterState optimization back after this bug in Qt 4.6 preview will be
+    // fixed (darktears promised it will get fixed in 4.6.0 release). Bug is about wrong coordinates
+    // for QPainter passed to QGS::drawForeground() function
     m_view->setOptimizationFlags( QGraphicsView::DontClipPainter |
-                                  QGraphicsView::DontSavePainterState |
+                                  //QGraphicsView::DontSavePainterState |
                                   QGraphicsView::DontAdjustForAntialiasing );
 
     top->addWidget(m_view, 1);
@@ -70,7 +74,7 @@ GameWidget::GameWidget ( int startingLevel, QWidget *parent )
     m_timer->setSingleShot(true);
     connect (m_timer, SIGNAL(timeout()), this, SLOT(nextLevel()));
 
-    switchToLevel(startingLevel);
+    setLevelSet(levelSet);
 }
 
 GameWidget::~GameWidget()
@@ -78,6 +82,42 @@ GameWidget::~GameWidget()
     delete m_highscore;
 }
 
+
+bool GameWidget::setLevelSet(const QString& levelSet)
+{
+    if (m_levelSet.name() == levelSet)
+    {
+        kDebug() << "level set named" << levelSet << "is already loaded";
+        return true;
+    }
+
+    bool res = m_levelSet.load(levelSet);
+    if (!res)
+    {
+        KMessageBox::error(this, i18n("Failed to load level set \"%1\". Check if it is installed on your computer.", levelSet));
+        kDebug() << "failed to load levelset" << levelSet;
+        return false;
+    }
+
+    int lastPlayed = lastPlayedLevel();
+    int maxLevel = maxAccessibleLevel();
+
+    int startingLevel = qMin(lastPlayed, maxLevel);
+    switchToLevel(startingLevel);
+
+    return true;
+}
+
+QString GameWidget::levelSetName() const
+{
+    return m_levelSet.name();
+}
+
+QString GameWidget::levelSetVisibleName() const
+{
+    return m_levelSet.visibleName();
+}
+
 QString GameWidget::currentMolecule() const
 {
     return m_playField->moleculeName();
@@ -110,21 +150,20 @@ void GameWidget::moveRight()
 void GameWidget::gameOver(int moves) {
     // writing this info only in normal mode
     if ( !m_allowAnyLevelSwitch &&
-		    Preferences::maxAccessibleLevel() < m_level+1 )
+         maxAccessibleLevel() < m_level+1 )
     {
-        Preferences::setMaxAccessibleLevel( m_level+1 );
-        Preferences::self()->writeConfig();
+        saveMaxAccessibleLevel( m_level+1 );
     }
 
     QString message = i18n( "Level %1 finished. ", m_level );
 
-    if( m_highscore->addScore( moves , m_level ) ) // new highscore!
+    if( m_highscore->addScore( moves , m_levelSet.name(), m_level ) ) // new highscore!
     {
         message += i18n("Congratulations! You have a new highscore!" );
     }
 
     m_playField->showMessage( message );
-    emit statsChanged(m_level, moves, m_highscore->levelHighscore(m_level));
+    emit statsChanged(m_level, moves, m_highscore->levelHighscore(m_levelSet.name(), m_level));
     if (!m_allowAnyLevelSwitch)
     {
         // reuse this signal to allow switching levels over toolbar
@@ -142,31 +181,22 @@ void GameWidget::updateMoves(int moves)
 
 void GameWidget::switchToLevel (int l)
 {
-    m_level=l;
-    const QString levelFile = KStandardDirs::locate("appdata", QString("levels/level_%1").arg(l));
-    if (levelFile.isNull()) {
-        if ( m_level != 1 )
-        {
-            KMessageBox::sorry( this,  i18n( "Level %1 can not be found. Please check your installation. Now switching to Level 1" ,  m_level ), "Broken installation?" );
-            switchToLevel(1);
-            return;
-        }
-        else if ( m_level == 1 )
-        {
-            KMessageBox::error( this,  i18n( "Level 1 data can not be found. Please check your installation. KAtomic will quit now." ) );
-            qApp->quit();
-            return;
-        }
+    const LevelData* levelData = m_levelSet.levelData(l);
+    if (levelData)
+    {
+        m_level=l;
+
+        m_playField->setLevelData(levelData);
+
+        m_levelHighscore = m_highscore->levelHighscore( m_levelSet.name(), m_level );
+
+        emit statsChanged(m_level, 0, m_levelHighscore);
+        emit levelChanged(m_level);
+
+        saveLastPlayedLevel();
     }
-
-    KConfig cfg( levelFile, KConfig::SimpleConfig);
-    KConfigGroup gr = cfg.group("Level");
-    m_playField->loadLevel(gr);
-
-    m_levelHighscore = m_highscore->levelHighscore( m_level );
-
-    emit statsChanged(m_level, 0, m_levelHighscore);
-    emit levelChanged(m_level);
+    else
+        kDebug() << "failed to load level" << l;
 }
 
 void GameWidget::restartLevel()
@@ -182,6 +212,7 @@ void GameWidget::saveGame()
     KConfig config(fileName, KConfig::SimpleConfig);
     KConfigGroup gr = config.group("Savegame");
     gr.writeEntry( "Level", m_level );
+    gr.writeEntry( "LevelSet", m_levelSet.name() );
     m_playField->saveGame( gr );
 }
 
@@ -192,10 +223,20 @@ void GameWidget::loadGame()
         return;
     KConfig config(fileName, KConfig::SimpleConfig);
     KConfigGroup gr = config.group("Savegame");
-    int l = gr.readEntry( "Level", 1 );
-    m_level = l;
-    switchToLevel(m_level);
-    m_playField->loadGame( gr );
+    QString levelSet = gr.readEntry("LevelSet");
+    if (levelSet.isEmpty())
+    {
+        kDebug() << "note: savegame file doesn't contain info about levelset, assuming default one";
+        levelSet = DEFAULT_LEVELSET_NAME;
+    }
+
+    bool res = setLevelSet(levelSet);
+    if (res)
+    {
+        int l = gr.readEntry( "Level", 1 );
+        switchToLevel(l);
+        m_playField->loadGame( gr );
+    }
 }
 
 void GameWidget::showHighscores ()
@@ -213,10 +254,11 @@ void GameWidget::prevLevel()
     // if user hits toolbar buttons, stop timer
     if (m_timer->isActive())
         m_timer->stop();
-    if ( m_level == 1 )
-        return;
-    m_level--;
-    switchToLevel(m_level);
+
+    if (isPrevLevelAvailable())
+    {
+        switchToLevel(m_level-1);
+    }
 }
 
 void GameWidget::nextLevel()
@@ -224,14 +266,11 @@ void GameWidget::nextLevel()
     // if user hits toolbar buttons, stop timer
     if (m_timer->isActive())
         m_timer->stop();
-    if ( !m_allowAnyLevelSwitch && m_level >= Preferences::maxAccessibleLevel() )
+
+    if (isNextLevelAvailable())
     {
-        KMessageBox::information(this, i18n("You must solve level %1 before advancing to the next one!",
-                                            Preferences::maxAccessibleLevel()), i18n("Warning"));
-        return;
+        switchToLevel(m_level+1);
     }
-    m_level++;
-    switchToLevel(m_level);
 }
 
 void GameWidget::resizeEvent( QResizeEvent* ev)
@@ -239,4 +278,50 @@ void GameWidget::resizeEvent( QResizeEvent* ev)
     m_playField->resize( ev->size().width(), ev->size().height() );
 }
 
+void GameWidget::saveLastPlayedLevel()
+{
+    KSharedConfigPtr cfg = KGlobal::config();
+    KConfigGroup grp(cfg, m_levelSet.name());
+    grp.writeEntry("LastPlayedLevel", m_level);
+
+    Preferences::setLastPlayedLevelSet(m_levelSet.name());
+}
+
+void GameWidget::saveMaxAccessibleLevel(int level)
+{
+    KSharedConfigPtr cfg = KGlobal::config();
+    KConfigGroup grp(cfg, m_levelSet.name());
+    grp.writeEntry("MaxAccessibleLevel", level);
+}
+
+int GameWidget::lastPlayedLevel() const
+{
+    KSharedConfigPtr cfg = KGlobal::config();
+    KConfigGroup grp(cfg, m_levelSet.name());
+    int lastPlayed = grp.readEntry("LastPlayedLevel", 1);
+    lastPlayed = qMax(1, lastPlayed); // can't be less than 1
+    kDebug() << "last played level:" << lastPlayed;
+    return lastPlayed;
+}
+
+int GameWidget::maxAccessibleLevel() const
+{
+    KSharedConfigPtr cfg = KGlobal::config();
+    KConfigGroup grp(cfg, m_levelSet.name());
+    int maxAccessible = grp.readEntry("MaxAccessibleLevel", 1);
+    kDebug() << "max accessible level:" << maxAccessible;
+    return maxAccessible;
+}
+
+bool GameWidget::isNextLevelAvailable() const
+{
+    bool avail = (m_level != maxAccessibleLevel() && m_level <= m_levelSet.levelCount());
+    return avail;
+}
+
+bool GameWidget::isPrevLevelAvailable() const
+{
+    return m_level > 1;
+}
+
 #include "gamewidget.moc"
